#!/bin/bash -x


# Wonder Shaper
# please read the README before filling out these values 
# true original here: http://lartc.org/wondershaper/
# by bert hubert <ahu@ds9a.nl>
# this version has some documentation added by hanscees.com 10-2005
#

#see what is happening with the command:
#tc -s class show dev eth1
# added stuff to make sure the upload from my local webserver does 
# not kill me surfing happily: 
#src port 80 low prio: from webserver to internet
#dst port 53 high prio: dns request from us to internet
#syns to internet high prio: we surf!o
#see 
#http://lartc.org/howto/lartc.adv-filter.html#LARTC.ADV-FILTER.U32





# Set the following values to somewhat less than your actual download
# and uplink speed. In kilobits. Also set the device that is to be shaped.

##these are values for adsl 1MB up and 2MB down.
DOWNLINK=1920
UPLINK=930
DEV=eth1

# low priority OUTGOING traffic - you can leave this blank if you want
# low priority source netmasks
NOPRIOHOSTSRC=

# low priority destination netmasks
NOPRIOHOSTDST=

# low priority source ports
NOPRIOPORTSRC=80

# low priority destination ports
NOPRIOPORTDST=1214


#high priority source ports (added by hanscees.com)
HIGHPRIOPORTDST=53


# Now remove the following two lines :-)


#########################################################

if [ "$1" = "status" ]
then
	tc -s qdisc ls dev $DEV
	tc -s class ls dev $DEV
	exit
fi


# clean existing down- and uplink qdiscs, hide errors
tc qdisc del dev $DEV root    2> /dev/null > /dev/null
tc qdisc del dev $DEV ingress 2> /dev/null > /dev/null

if [ "$1" = "stop" ] 
then 
	exit
fi

###### uplink

# install root CBQ

tc qdisc add dev $DEV root handle 1: cbq avpkt 1000 bandwidth 10mbit 

# shape everything at $UPLINK speed - this prevents huge queues in your
# DSL modem which destroy latency:
# main class

tc class add dev $DEV parent 1: classid 1:1 cbq rate ${UPLINK}kbit \
allot 1500 prio 5 bounded isolated 

# high prio class 1:10:

tc class add dev $DEV parent 1:1 classid 1:10 cbq rate ${UPLINK}kbit \
   allot 1600 prio 1 avpkt 1000

# bulk and default class 1:20 - gets slightly less traffic, 
#  and a lower priority:

tc class add dev $DEV parent 1:1 classid 1:20 cbq rate $[9*$UPLINK/10]kbit \
   allot 1600 prio 2 avpkt 1000

# 'traffic we hate'

tc class add dev $DEV parent 1:1 classid 1:30 cbq rate $[8*$UPLINK/10]kbit \
   allot 1600 prio 2 avpkt 1000

# all get Stochastic Fairness:
tc qdisc add dev $DEV parent 1:10 handle 10: sfq perturb 10
tc qdisc add dev $DEV parent 1:20 handle 20: sfq perturb 10
tc qdisc add dev $DEV parent 1:30 handle 30: sfq perturb 10

# start filters
# TOS Minimum Delay (ssh, NOT scp) in 1:10:
tc filter add dev $DEV parent 1:0 protocol ip prio 10 u32 \
      match ip tos 0x10 0xff  flowid 1:10

# ICMP (ip protocol 1) in the interactive class 1:10 so we 
# can do measurements & impress our friends:
tc filter add dev $DEV parent 1:0 protocol ip prio 11 u32 \
        match ip protocol 1 0xff flowid 1:10

# prioritize small packets (<64 bytes)
#and ack packets

tc filter add dev $DEV parent 1: protocol ip prio 12 u32 \
   match ip protocol 6 0xff \
   match u8 0x05 0x0f at 0 \
   match u16 0x0000 0xffc0 at 2 \
   match u8 0x10 0xff at 33 \
   flowid 1:10

   #match u8 0x10 0xff at 33 \ added by hansces.com
   #see 
   #http://lartc.org/howto/lartc.adv-filter.html#LARTC.ADV-FILTER.U32

#  add some ports that are high prio
#HIGHPRIOPORTDST=53 by hancees.com
for a in $HIGHPRIOPORTDST
do
 	tc filter add dev $DEV parent 1: protocol ip prio 13 u32 \
	   match ip dport $a 0xffff flowid 1:10
done

# some traffic however suffers a worse fate
for a in $NOPRIOPORTDST
do
	tc filter add dev $DEV parent 1: protocol ip prio 14 u32 \
	   match ip dport $a 0xffff flowid 1:30
done

for a in $NOPRIOPORTSRC
do
 	tc filter add dev $DEV parent 1: protocol ip prio 15 u32 \
	   match ip sport $a 0xffff flowid 1:30
done

for a in $NOPRIOHOSTSRC
do
 	tc filter add dev $DEV parent 1: protocol ip prio 16 u32 \
	   match ip src $a flowid 1:30
done

for a in $NOPRIOHOSTDST
do
 	tc filter add dev $DEV parent 1: protocol ip prio 17 u32 \
	   match ip dst $a flowid 1:30
done

# rest is 'non-interactive' ie 'bulk' and ends up in 1:20

tc filter add dev $DEV parent 1: protocol ip prio 18 u32 \
   match ip dst 0.0.0.0/0 flowid 1:20


########## downlink #############
#from http://lartc.org/howto/lartc.qdisc.terminology.html
#                Userspace programs
#                     ^
#                     |
#     +---------------+-----------------------------------------+
#     |               Y                                         |
#     |    -------> IP Stack                                    |
#     |   |              |                                      |
#     |   |              Y                                      |
#     |   |              Y                                      |
#     |   ^              |                                      |
#     |   |  / ----------> Forwarding ->                        |
#     |   ^ /                           |                       |
#     |   |/                            Y                       |
#     |   |                             |                       |
#     |   ^                             Y          /-qdisc1-\   |
#     |   |                            Egress     /--qdisc2--\  |
#  --->->Ingress                       Classifier ---qdisc3---- | ->
#     |   Qdisc                                   \__qdisc4__/  |
#     |                                            \-qdiscN_/   |
#     |                                                         |
#     +----------------------------------------------------------+
#Thanks to Jamal Hadi Salim for this ASCII representation.
#
#The big block represents the kernel. The leftmost arrow represents traffic entering your machine from the #network. It is then fed to the Ingress Qdisc which may apply Filters to a packet, and decide to drop it. #This is called 'Policing'.
#This happens at a very early stage, before it has seen a lot of the kernel. It is therefore a very good #place to drop traffic very early, without consuming a lot of CPU power.

# slow downloads down to somewhat less than the real speed  to prevent 
# queuing at our ISP. Tune to see how high you can set it.
# ISPs tend to have *huge* queues to make sure big downloads are fast
#
# attach ingress policer:


tc qdisc add dev $DEV handle ffff: ingress

# filter *everything* to it (0.0.0.0/0), drop everything that's
# coming in too fast:

tc filter add dev $DEV parent ffff: protocol ip prio 50 u32 match ip src \
   0.0.0.0/0 police rate ${DOWNLINK}kbit burst 10k drop flowid :1

   logger "wshaper finished"
